/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(NETSCAPE_PLUGIN_API)
#include "NPObjectMessageReceiver.h"

#include "ArgumentCoders.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "NPIdentifierData.h"
#include "NPObjectMessageReceiverMessages.h"
#include "NPVariantData.h"
#include <wtf/Vector.h>

namespace Messages {

namespace NPObjectMessageReceiver {

void Deallocate::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

void HasMethod::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue)
{
    encoder.get() << returnValue;
    connection.sendSyncReply(WTFMove(encoder));
}

void Invoke::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue, const WebKit::NPVariantData& resultData)
{
    encoder.get() << returnValue;
    encoder.get() << resultData;
    connection.sendSyncReply(WTFMove(encoder));
}

void InvokeDefault::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue, const WebKit::NPVariantData& resultData)
{
    encoder.get() << returnValue;
    encoder.get() << resultData;
    connection.sendSyncReply(WTFMove(encoder));
}

void HasProperty::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue)
{
    encoder.get() << returnValue;
    connection.sendSyncReply(WTFMove(encoder));
}

void GetProperty::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue, const WebKit::NPVariantData& resultData)
{
    encoder.get() << returnValue;
    encoder.get() << resultData;
    connection.sendSyncReply(WTFMove(encoder));
}

void SetProperty::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue)
{
    encoder.get() << returnValue;
    connection.sendSyncReply(WTFMove(encoder));
}

void RemoveProperty::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue)
{
    encoder.get() << returnValue;
    connection.sendSyncReply(WTFMove(encoder));
}

void Enumerate::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue, const Vector<WebKit::NPIdentifierData>& identifiersData)
{
    encoder.get() << returnValue;
    encoder.get() << identifiersData;
    connection.sendSyncReply(WTFMove(encoder));
}

void Construct::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue, const WebKit::NPVariantData& resultData)
{
    encoder.get() << returnValue;
    encoder.get() << resultData;
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace NPObjectMessageReceiver

} // namespace Messages

namespace WebKit {


bool NPObjectMessageReceiver::didReceiveSyncNPObjectMessageReceiverMessage(IPC::Connection& connection, IPC::Decoder& decoder, UniqueRef<IPC::Encoder>& replyEncoder)
{
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::Deallocate::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::Deallocate>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::deallocate);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::HasMethod::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::HasMethod>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::hasMethod);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::Invoke::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::Invoke>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::invoke);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::InvokeDefault::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::InvokeDefault>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::invokeDefault);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::HasProperty::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::HasProperty>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::hasProperty);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::GetProperty::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::GetProperty>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::getProperty);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::SetProperty::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::SetProperty>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::setProperty);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::RemoveProperty::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::RemoveProperty>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::removeProperty);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::Enumerate::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::Enumerate>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::enumerate);
    if (decoder.messageName() == Messages::NPObjectMessageReceiver::Construct::name())
        return IPC::handleMessageSynchronous<Messages::NPObjectMessageReceiver::Construct>(connection, decoder, replyEncoder, this, &NPObjectMessageReceiver::construct);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return false;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled synchronous message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
    return false;
}

} // namespace WebKit

#endif // ENABLE(NETSCAPE_PLUGIN_API)
