/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(NETSCAPE_PLUGIN_API)
#include "PluginControllerProxy.h"

#include "ArgumentCoders.h"
#include "DataReference.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "PluginControllerProxyMessages.h"
#include "ShareableBitmap.h"
#include "WebCoreArgumentCoders.h"
#include "WebKeyboardEvent.h"
#include "WebMouseEvent.h"
#include "WebWheelEvent.h"
#include <WebCore/AffineTransform.h>
#include <WebCore/IntRect.h>
#include <WebCore/IntSize.h>
#include <wtf/text/WTFString.h>

namespace Messages {

namespace PluginControllerProxy {

void HandleWheelEvent::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool handled)
{
    encoder.get() << handled;
    connection.sendSyncReply(WTFMove(encoder));
}

void HandleMouseEnterEvent::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool handled)
{
    encoder.get() << handled;
    connection.sendSyncReply(WTFMove(encoder));
}

void HandleMouseLeaveEvent::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool handled)
{
    encoder.get() << handled;
    connection.sendSyncReply(WTFMove(encoder));
}

void HandleKeyboardEvent::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool handled)
{
    encoder.get() << handled;
    connection.sendSyncReply(WTFMove(encoder));
}

void HandleEditingCommand::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool handled)
{
    encoder.get() << handled;
    connection.sendSyncReply(WTFMove(encoder));
}

void IsEditingCommandEnabled::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool enabled)
{
    encoder.get() << enabled;
    connection.sendSyncReply(WTFMove(encoder));
}

void HandlesPageScaleFactor::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool enabled)
{
    encoder.get() << enabled;
    connection.sendSyncReply(WTFMove(encoder));
}

void RequiresUnifiedScaleFactor::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool required)
{
    encoder.get() << required;
    connection.sendSyncReply(WTFMove(encoder));
}

void PaintEntirePlugin::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

void GetPluginScriptableNPObject::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, uint64_t pluginScriptableNPObjectID)
{
    encoder.get() << pluginScriptableNPObjectID;
    connection.sendSyncReply(WTFMove(encoder));
}

void SupportsSnapshotting::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool isSupported)
{
    encoder.get() << isSupported;
    connection.sendSyncReply(WTFMove(encoder));
}

void Snapshot::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebKit::ShareableBitmap::Handle& backingStoreHandle)
{
    encoder.get() << backingStoreHandle;
    connection.sendSyncReply(WTFMove(encoder));
}

void GetFormValue::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool returnValue, const String& formValue)
{
    encoder.get() << returnValue;
    encoder.get() << formValue;
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace PluginControllerProxy

} // namespace Messages

namespace WebKit {

void PluginControllerProxy::didReceivePluginControllerProxyMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::PluginControllerProxy::GeometryDidChange::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::GeometryDidChange>(decoder, this, &PluginControllerProxy::geometryDidChange);
    if (decoder.messageName() == Messages::PluginControllerProxy::VisibilityDidChange::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::VisibilityDidChange>(decoder, this, &PluginControllerProxy::visibilityDidChange);
    if (decoder.messageName() == Messages::PluginControllerProxy::FrameDidFinishLoading::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::FrameDidFinishLoading>(decoder, this, &PluginControllerProxy::frameDidFinishLoading);
    if (decoder.messageName() == Messages::PluginControllerProxy::FrameDidFail::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::FrameDidFail>(decoder, this, &PluginControllerProxy::frameDidFail);
    if (decoder.messageName() == Messages::PluginControllerProxy::DidEvaluateJavaScript::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::DidEvaluateJavaScript>(decoder, this, &PluginControllerProxy::didEvaluateJavaScript);
    if (decoder.messageName() == Messages::PluginControllerProxy::StreamWillSendRequest::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::StreamWillSendRequest>(decoder, this, &PluginControllerProxy::streamWillSendRequest);
    if (decoder.messageName() == Messages::PluginControllerProxy::StreamDidReceiveResponse::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::StreamDidReceiveResponse>(decoder, this, &PluginControllerProxy::streamDidReceiveResponse);
    if (decoder.messageName() == Messages::PluginControllerProxy::StreamDidReceiveData::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::StreamDidReceiveData>(decoder, this, &PluginControllerProxy::streamDidReceiveData);
    if (decoder.messageName() == Messages::PluginControllerProxy::StreamDidFinishLoading::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::StreamDidFinishLoading>(decoder, this, &PluginControllerProxy::streamDidFinishLoading);
    if (decoder.messageName() == Messages::PluginControllerProxy::StreamDidFail::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::StreamDidFail>(decoder, this, &PluginControllerProxy::streamDidFail);
    if (decoder.messageName() == Messages::PluginControllerProxy::ManualStreamDidReceiveResponse::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::ManualStreamDidReceiveResponse>(decoder, this, &PluginControllerProxy::manualStreamDidReceiveResponse);
    if (decoder.messageName() == Messages::PluginControllerProxy::ManualStreamDidReceiveData::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::ManualStreamDidReceiveData>(decoder, this, &PluginControllerProxy::manualStreamDidReceiveData);
    if (decoder.messageName() == Messages::PluginControllerProxy::ManualStreamDidFinishLoading::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::ManualStreamDidFinishLoading>(decoder, this, &PluginControllerProxy::manualStreamDidFinishLoading);
    if (decoder.messageName() == Messages::PluginControllerProxy::ManualStreamDidFail::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::ManualStreamDidFail>(decoder, this, &PluginControllerProxy::manualStreamDidFail);
    if (decoder.messageName() == Messages::PluginControllerProxy::HandleMouseEvent::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::HandleMouseEvent>(decoder, this, &PluginControllerProxy::handleMouseEvent);
    if (decoder.messageName() == Messages::PluginControllerProxy::SetFocus::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::SetFocus>(decoder, this, &PluginControllerProxy::setFocus);
    if (decoder.messageName() == Messages::PluginControllerProxy::DidUpdate::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::DidUpdate>(decoder, this, &PluginControllerProxy::didUpdate);
    if (decoder.messageName() == Messages::PluginControllerProxy::WindowFocusChanged::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::WindowFocusChanged>(decoder, this, &PluginControllerProxy::windowFocusChanged);
    if (decoder.messageName() == Messages::PluginControllerProxy::WindowVisibilityChanged::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::WindowVisibilityChanged>(decoder, this, &PluginControllerProxy::windowVisibilityChanged);
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginControllerProxy::SendComplexTextInput::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::SendComplexTextInput>(decoder, this, &PluginControllerProxy::sendComplexTextInput);
#endif
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginControllerProxy::WindowAndViewFramesChanged::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::WindowAndViewFramesChanged>(decoder, this, &PluginControllerProxy::windowAndViewFramesChanged);
#endif
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginControllerProxy::SetLayerHostingMode::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::SetLayerHostingMode>(decoder, this, &PluginControllerProxy::setLayerHostingMode);
#endif
    if (decoder.messageName() == Messages::PluginControllerProxy::StorageBlockingStateChanged::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::StorageBlockingStateChanged>(decoder, this, &PluginControllerProxy::storageBlockingStateChanged);
    if (decoder.messageName() == Messages::PluginControllerProxy::PrivateBrowsingStateChanged::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::PrivateBrowsingStateChanged>(decoder, this, &PluginControllerProxy::privateBrowsingStateChanged);
    if (decoder.messageName() == Messages::PluginControllerProxy::MutedStateChanged::name())
        return IPC::handleMessage<Messages::PluginControllerProxy::MutedStateChanged>(decoder, this, &PluginControllerProxy::mutedStateChanged);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
}

bool PluginControllerProxy::didReceiveSyncPluginControllerProxyMessage(IPC::Connection& connection, IPC::Decoder& decoder, UniqueRef<IPC::Encoder>& replyEncoder)
{
    if (decoder.messageName() == Messages::PluginControllerProxy::HandleWheelEvent::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::HandleWheelEvent>(connection, decoder, replyEncoder, this, &PluginControllerProxy::handleWheelEvent);
    if (decoder.messageName() == Messages::PluginControllerProxy::HandleMouseEnterEvent::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::HandleMouseEnterEvent>(connection, decoder, replyEncoder, this, &PluginControllerProxy::handleMouseEnterEvent);
    if (decoder.messageName() == Messages::PluginControllerProxy::HandleMouseLeaveEvent::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::HandleMouseLeaveEvent>(connection, decoder, replyEncoder, this, &PluginControllerProxy::handleMouseLeaveEvent);
    if (decoder.messageName() == Messages::PluginControllerProxy::HandleKeyboardEvent::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::HandleKeyboardEvent>(connection, decoder, replyEncoder, this, &PluginControllerProxy::handleKeyboardEvent);
    if (decoder.messageName() == Messages::PluginControllerProxy::HandleEditingCommand::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::HandleEditingCommand>(connection, decoder, replyEncoder, this, &PluginControllerProxy::handleEditingCommand);
    if (decoder.messageName() == Messages::PluginControllerProxy::IsEditingCommandEnabled::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::IsEditingCommandEnabled>(connection, decoder, replyEncoder, this, &PluginControllerProxy::isEditingCommandEnabled);
    if (decoder.messageName() == Messages::PluginControllerProxy::HandlesPageScaleFactor::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::HandlesPageScaleFactor>(connection, decoder, replyEncoder, this, &PluginControllerProxy::handlesPageScaleFactor);
    if (decoder.messageName() == Messages::PluginControllerProxy::RequiresUnifiedScaleFactor::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::RequiresUnifiedScaleFactor>(connection, decoder, replyEncoder, this, &PluginControllerProxy::requiresUnifiedScaleFactor);
    if (decoder.messageName() == Messages::PluginControllerProxy::PaintEntirePlugin::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::PaintEntirePlugin>(connection, decoder, replyEncoder, this, &PluginControllerProxy::paintEntirePlugin);
    if (decoder.messageName() == Messages::PluginControllerProxy::GetPluginScriptableNPObject::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::GetPluginScriptableNPObject>(connection, decoder, replyEncoder, this, &PluginControllerProxy::getPluginScriptableNPObject);
    if (decoder.messageName() == Messages::PluginControllerProxy::SupportsSnapshotting::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::SupportsSnapshotting>(connection, decoder, replyEncoder, this, &PluginControllerProxy::supportsSnapshotting);
    if (decoder.messageName() == Messages::PluginControllerProxy::Snapshot::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::Snapshot>(connection, decoder, replyEncoder, this, &PluginControllerProxy::snapshot);
    if (decoder.messageName() == Messages::PluginControllerProxy::GetFormValue::name())
        return IPC::handleMessageSynchronous<Messages::PluginControllerProxy::GetFormValue>(connection, decoder, replyEncoder, this, &PluginControllerProxy::getFormValue);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return false;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled synchronous message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
    return false;
}

} // namespace WebKit

#endif // ENABLE(NETSCAPE_PLUGIN_API)
