/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "RemoteMediaEngineConfigurationFactoryProxy.h"

#include "Decoder.h"
#include "HandleMessage.h"
#include "RemoteMediaEngineConfigurationFactoryProxyMessages.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/MediaCapabilitiesDecodingInfo.h>
#include <WebCore/MediaCapabilitiesEncodingInfo.h>
#include <WebCore/MediaDecodingConfiguration.h>
#include <WebCore/MediaEncodingConfiguration.h>

namespace Messages {

namespace RemoteMediaEngineConfigurationFactoryProxy {

void CreateDecodingConfiguration::callReply(IPC::Decoder& decoder, CompletionHandler<void(WebCore::MediaCapabilitiesDecodingInfo&&)>&& completionHandler)
{
    std::optional<WebCore::MediaCapabilitiesDecodingInfo> decodingInfo;
    decoder >> decodingInfo;
    if (!decodingInfo) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*decodingInfo));
}

void CreateDecodingConfiguration::cancelReply(CompletionHandler<void(WebCore::MediaCapabilitiesDecodingInfo&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<WebCore::MediaCapabilitiesDecodingInfo>::create());
}

void CreateDecodingConfiguration::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebCore::MediaCapabilitiesDecodingInfo& decodingInfo)
{
    encoder.get() << decodingInfo;
    connection.sendSyncReply(WTFMove(encoder));
}

void CreateEncodingConfiguration::callReply(IPC::Decoder& decoder, CompletionHandler<void(WebCore::MediaCapabilitiesEncodingInfo&&)>&& completionHandler)
{
    std::optional<WebCore::MediaCapabilitiesEncodingInfo> decodingInfo;
    decoder >> decodingInfo;
    if (!decodingInfo) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*decodingInfo));
}

void CreateEncodingConfiguration::cancelReply(CompletionHandler<void(WebCore::MediaCapabilitiesEncodingInfo&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<WebCore::MediaCapabilitiesEncodingInfo>::create());
}

void CreateEncodingConfiguration::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebCore::MediaCapabilitiesEncodingInfo& decodingInfo)
{
    encoder.get() << decodingInfo;
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace RemoteMediaEngineConfigurationFactoryProxy

} // namespace Messages

namespace WebKit {

void RemoteMediaEngineConfigurationFactoryProxy::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::RemoteMediaEngineConfigurationFactoryProxy::CreateDecodingConfiguration::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaEngineConfigurationFactoryProxy::CreateDecodingConfiguration>(connection, decoder, this, &RemoteMediaEngineConfigurationFactoryProxy::createDecodingConfiguration);
    if (decoder.messageName() == Messages::RemoteMediaEngineConfigurationFactoryProxy::CreateEncodingConfiguration::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaEngineConfigurationFactoryProxy::CreateEncodingConfiguration>(connection, decoder, this, &RemoteMediaEngineConfigurationFactoryProxy::createEncodingConfiguration);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS)
