/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if PLATFORM(COCOA) && ENABLE(GPU_PROCESS) && ENABLE(MEDIA_STREAM) && HAVE(AVASSETWRITERDELEGATE)
#include "RemoteMediaRecorderManager.h"

#include "ArgumentCoders.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "MediaRecorderIdentifier.h"
#include "RemoteMediaRecorderManagerMessages.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/ExceptionData.h>
#include <WebCore/MediaRecorderPrivateOptions.h>
#include <optional>
#include <wtf/text/WTFString.h>

namespace Messages {

namespace RemoteMediaRecorderManager {

void CreateRecorder::callReply(IPC::Decoder& decoder, CompletionHandler<void(std::optional<WebCore::ExceptionData>&&, String&&, unsigned&&, unsigned&&)>&& completionHandler)
{
    std::optional<std::optional<WebCore::ExceptionData>> creationError;
    decoder >> creationError;
    if (!creationError) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    std::optional<String> mimeType;
    decoder >> mimeType;
    if (!mimeType) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    std::optional<unsigned> audioBitRate;
    decoder >> audioBitRate;
    if (!audioBitRate) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    std::optional<unsigned> videoBitRate;
    decoder >> videoBitRate;
    if (!videoBitRate) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*creationError), WTFMove(*mimeType), WTFMove(*audioBitRate), WTFMove(*videoBitRate));
}

void CreateRecorder::cancelReply(CompletionHandler<void(std::optional<WebCore::ExceptionData>&&, String&&, unsigned&&, unsigned&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<std::optional<WebCore::ExceptionData>>::create(), IPC::AsyncReplyError<String>::create(), IPC::AsyncReplyError<unsigned>::create(), IPC::AsyncReplyError<unsigned>::create());
}

void CreateRecorder::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const std::optional<WebCore::ExceptionData>& creationError, const String& mimeType, const unsigned& audioBitRate, const unsigned& videoBitRate)
{
    encoder.get() << creationError;
    encoder.get() << mimeType;
    encoder.get() << audioBitRate;
    encoder.get() << videoBitRate;
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace RemoteMediaRecorderManager

} // namespace Messages

namespace WebKit {

void RemoteMediaRecorderManager::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::RemoteMediaRecorderManager::CreateRecorder::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaRecorderManager::CreateRecorder>(connection, decoder, this, &RemoteMediaRecorderManager::createRecorder);
    if (decoder.messageName() == Messages::RemoteMediaRecorderManager::ReleaseRecorder::name())
        return IPC::handleMessage<Messages::RemoteMediaRecorderManager::ReleaseRecorder>(decoder, this, &RemoteMediaRecorderManager::releaseRecorder);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
}

} // namespace WebKit

#endif // PLATFORM(COCOA) && ENABLE(GPU_PROCESS) && ENABLE(MEDIA_STREAM) && HAVE(AVASSETWRITERDELEGATE)
